@@ -25,33 +25,60 @@ if (isTRUE(VCD_AVAILABLE)) {
     label <- df[, ifelse(Improved == "Marked", 1, 0)]
 
     # binary
-    bst.Tree <- xgb.train(data = xgb.DMatrix(sparse_matrix, label = label), max_depth = 9,
-                          eta = 1, nthread = 2, nrounds = nrounds, verbose = 0,
-                          objective = "binary:logistic", booster = "gbtree",
-                          base_score = 0.5)
+    bst.Tree <- xgb.train(
+      data = xgb.DMatrix(sparse_matrix, label = label, nthread = 1),
+      nrounds = nrounds, verbose = 0,
+      params = xgb.params(
+        max_depth = 9,
+        learning_rate = 1,
+        nthread = 2,
+        objective = "binary:logistic",
+        booster = "gbtree",
+        base_score = 0.5
+      )
+    )
 
-    bst.GLM <- xgb.train(data = xgb.DMatrix(sparse_matrix, label = label),
-                         eta = 1, nthread = 1, nrounds = nrounds, verbose = 0,
-                         objective = "binary:logistic", booster = "gblinear",
-                         base_score = 0.5)
+    bst.GLM <- xgb.train(
+      data = xgb.DMatrix(sparse_matrix, label = label, nthread = 1),
+      nrounds = nrounds, verbose = 0,
+      params = xgb.params(
+        learning_rate = 1,
+        nthread = 1,
+        objective = "binary:logistic",
+        booster = "gblinear",
+        base_score = 0.5
+      )
+    )
 
     feature.names <- colnames(sparse_matrix)
+
+    # without feature names
+    bst.Tree.unnamed <- xgb.copy.Booster(bst.Tree)
+    setinfo(bst.Tree.unnamed, "feature_name", NULL)
 }
 
 # multiclass
 mlabel <- as.numeric(iris$Species) - 1
 nclass <- 3
-mbst.Tree <- xgb.train(data = xgb.DMatrix(as.matrix(iris[, -5]), label = mlabel), verbose = 0,
-                       max_depth = 3, eta = 0.5, nthread = 2, nrounds = nrounds,
-                       objective = "multi:softprob", num_class = nclass, base_score = 0)
+mbst.Tree <- xgb.train(
+  data = xgb.DMatrix(as.matrix(iris[, -5]), label = mlabel, nthread = 1),
+  verbose = 0,
+  nrounds = nrounds,
+  params = xgb.params(
+    max_depth = 3, learning_rate = 0.5, nthread = 2,
+    objective = "multi:softprob", num_class = nclass, base_score = 0
+  )
+)
 
-mbst.GLM <- xgb.train(data = xgb.DMatrix(as.matrix(iris[, -5]), label = mlabel), verbose = 0,
-                      booster = "gblinear", eta = 0.1, nthread = 1, nrounds = nrounds,
-                      objective = "multi:softprob", num_class = nclass, base_score = 0)
-
-# without feature names
-bst.Tree.unnamed <- xgb.copy.Booster(bst.Tree)
-setinfo(bst.Tree.unnamed, "feature_name", NULL)
+mbst.GLM <- xgb.train(
+  data = xgb.DMatrix(as.matrix(iris[, -5]), label = mlabel, nthread = 1),
+  verbose = 0,
+  nrounds = nrounds,
+  params = xgb.params(
+    booster = "gblinear", learning_rate = 0.1, nthread = 1,
+    objective = "multi:softprob", num_class = nclass, base_score = 0
+  )
+)
 
 test_that("xgb.dump works", {
   .skip_if_vcd_not_available()
@@ -74,9 +101,17 @@ test_that("xgb.dump works for gblinear", {
   expect_length(xgb.dump(bst.GLM), 14)
   # also make sure that it works properly for a sparse model where some coefficients
   # are 0 from setting large L1 regularization:
-  bst.GLM.sp <- xgb.train(data = xgb.DMatrix(sparse_matrix, label = label), eta = 1,
-                          nthread = 2, nrounds = 1,
-                          alpha = 2, objective = "binary:logistic", booster = "gblinear")
+  bst.GLM.sp <- xgb.train(
+    data = xgb.DMatrix(sparse_matrix, label = label, nthread = 1),
+    nrounds = 1,
+    params = xgb.params(
+      learning_rate = 1,
+      nthread = 2,
+      reg_alpha = 2,
+      objective = "binary:logistic",
+      booster = "gblinear"
+    )
+  )
   d.sp <- xgb.dump(bst.GLM.sp)
   expect_length(d.sp, 14)
   expect_gt(sum(d.sp == "0"), 0)
@@ -101,7 +136,7 @@ test_that("predict feature contributions works", {
   # gbtree binary classifier
   expect_error(pred_contr <- predict(bst.Tree, sparse_matrix, predcontrib = TRUE), regexp = NA)
   expect_equal(dim(pred_contr), c(nrow(sparse_matrix), ncol(sparse_matrix) + 1))
-  expect_equal(colnames(pred_contr), c(colnames(sparse_matrix), "BIAS"))
+  expect_equal(colnames(pred_contr), c(colnames(sparse_matrix), "(Intercept)"))
   pred <- predict(bst.Tree, sparse_matrix, outputmargin = TRUE)
   expect_lt(max(abs(rowSums(pred_contr) - pred)), 1e-5)
   # must work with data that has no column names
@@ -114,14 +149,14 @@ test_that("predict feature contributions works", {
   # gbtree binary classifier (approximate method)
   expect_error(pred_contr <- predict(bst.Tree, sparse_matrix, predcontrib = TRUE, approxcontrib = TRUE), regexp = NA)
   expect_equal(dim(pred_contr), c(nrow(sparse_matrix), ncol(sparse_matrix) + 1))
-  expect_equal(colnames(pred_contr), c(colnames(sparse_matrix), "BIAS"))
+  expect_equal(colnames(pred_contr), c(colnames(sparse_matrix), "(Intercept)"))
   pred <- predict(bst.Tree, sparse_matrix, outputmargin = TRUE)
   expect_lt(max(abs(rowSums(pred_contr) - pred)), 1e-5)
 
   # gblinear binary classifier
   expect_error(pred_contr <- predict(bst.GLM, sparse_matrix, predcontrib = TRUE), regexp = NA)
   expect_equal(dim(pred_contr), c(nrow(sparse_matrix), ncol(sparse_matrix) + 1))
-  expect_equal(colnames(pred_contr), c(colnames(sparse_matrix), "BIAS"))
+  expect_equal(colnames(pred_contr), c(colnames(sparse_matrix), "(Intercept)"))
   pred <- predict(bst.GLM, sparse_matrix, outputmargin = TRUE)
   expect_lt(max(abs(rowSums(pred_contr) - pred)), 1e-5)
   # manual calculation of linear terms
@@ -132,31 +167,31 @@ test_that("predict feature contributions works", {
                tolerance = float_tolerance)
 
   # gbtree multiclass
-  pred <- predict(mbst.Tree, as.matrix(iris[, -5]), outputmargin = TRUE, reshape = TRUE)
+  pred <- predict(mbst.Tree, as.matrix(iris[, -5]), outputmargin = TRUE)
   pred_contr <- predict(mbst.Tree, as.matrix(iris[, -5]), predcontrib = TRUE)
-  expect_is(pred_contr, "list")
-  expect_length(pred_contr, 3)
-  for (g in seq_along(pred_contr)) {
-    expect_equal(colnames(pred_contr[[g]]), c(colnames(iris[, -5]), "BIAS"))
-    expect_lt(max(abs(rowSums(pred_contr[[g]]) - pred[, g])), 1e-5)
+  expect_is(pred_contr, "array")
+  expect_length(dim(pred_contr), 3)
+  for (g in seq_len(dim(pred_contr)[2])) {
+    expect_equal(colnames(pred_contr[, g, ]), c(colnames(iris[, -5]), "(Intercept)"))
+    expect_lt(max(abs(rowSums(pred_contr[, g, ]) - pred[, g])), 1e-5)
   }
 
   # gblinear multiclass (set base_score = 0, which is base margin in multiclass)
-  pred <- predict(mbst.GLM, as.matrix(iris[, -5]), outputmargin = TRUE, reshape = TRUE)
+  pred <- predict(mbst.GLM, as.matrix(iris[, -5]), outputmargin = TRUE)
   pred_contr <- predict(mbst.GLM, as.matrix(iris[, -5]), predcontrib = TRUE)
-  expect_length(pred_contr, 3)
+  expect_length(dim(pred_contr), 3)
   coefs_all <- matrix(
     data = as.numeric(xgb.dump(mbst.GLM)[-c(1, 2, 6)]),
     ncol = 3,
     byrow = TRUE
   )
-  for (g in seq_along(pred_contr)) {
-    expect_equal(colnames(pred_contr[[g]]), c(colnames(iris[, -5]), "BIAS"))
-    expect_lt(max(abs(rowSums(pred_contr[[g]]) - pred[, g])), float_tolerance)
+  for (g in seq_along(dim(pred_contr)[2])) {
+    expect_equal(colnames(pred_contr[, g, ]), c(colnames(iris[, -5]), "(Intercept)"))
+    expect_lt(max(abs(rowSums(pred_contr[, g, ]) - pred[, g])), float_tolerance)
     # manual calculation of linear terms
     coefs <- c(coefs_all[-1, g], coefs_all[1, g]) # intercept needs to be the last
     pred_contr_manual <- sweep(as.matrix(cbind(iris[, -5], 1)), 2, coefs, FUN = "*")
-    expect_equal(as.numeric(pred_contr[[g]]), as.numeric(pred_contr_manual),
+    expect_equal(as.numeric(pred_contr[, g, ]), as.numeric(pred_contr_manual),
                  tolerance = float_tolerance)
   }
 })
@@ -181,7 +216,7 @@ test_that("SHAPs sum to predictions, with or without DART", {
           eval_metric = "rmse"),
         if (booster == "dart")
           list(rate_drop = .01, one_drop = TRUE)),
-      data = xgb.DMatrix(d, label = y),
+      data = xgb.DMatrix(d, label = y, nthread = 1),
       nrounds = nrounds)
 
     pr <- function(...) {
@@ -323,13 +358,12 @@ test_that("xgb.importance works with and without feature names", {
   imp.Tree <- xgb.importance(model = mbst.Tree)
   expect_equal(dim(imp.Tree), c(4, 4))
 
-  trees <- seq(from = 0, by = 2, length.out = 2)
+  trees <- seq(from = 1, by = 2, length.out = 2)
   importance <- xgb.importance(feature_names = feature.names, model = bst.Tree, trees = trees)
 
   importance_from_dump <- function() {
-    model_text_dump <- xgb.dump(model = bst.Tree, with_stats = TRUE, trees = trees)
     imp <- xgb.model.dt.tree(
-      text = model_text_dump,
+      model = bst.Tree,
       trees = trees
     )[
       Feature != "Leaf", .(
@@ -352,11 +386,13 @@ test_that("xgb.importance works with and without feature names", {
   expect_equal(importance_from_dump(), importance, tolerance = 1e-6)
 
   ## decision stump
-  m <- xgboost::xgb.train(
-    data = xgb.DMatrix(as.matrix(data.frame(x = c(0, 1))), label = c(1, 2)),
+  m <- xgb.train(
+    data = xgb.DMatrix(as.matrix(data.frame(x = c(0, 1))), label = c(1, 2), nthread = 1),
     nrounds = 1,
-    base_score = 0.5,
-    nthread = 2
+    params = xgb.params(
+      base_score = 0.5,
+      nthread = 2
+    )
   )
   df <- xgb.model.dt.tree(model = m)
   expect_equal(df$Feature, "Leaf")
@@ -384,9 +420,16 @@ test_that("xgb.importance works with GLM model", {
 
 test_that("xgb.model.dt.tree and xgb.importance work with a single split model", {
   .skip_if_vcd_not_available()
-  bst1 <- xgb.train(data = xgb.DMatrix(sparse_matrix, label = label), max_depth = 1,
-                    eta = 1, nthread = 2, nrounds = 1, verbose = 0,
-                    objective = "binary:logistic")
+  bst1 <- xgb.train(
+    data = xgb.DMatrix(sparse_matrix, label = label, nthread = 1),
+    nrounds = 1, verbose = 0,
+    params = xgb.params(
+      max_depth = 1,
+      learning_rate = 1,
+      nthread = 2,
+      objective = "binary:logistic"
+    )
+  )
   expect_error(dt <- xgb.model.dt.tree(model = bst1), regexp = NA) # no error
   expect_equal(nrow(dt), 3)
   expect_error(imp <- xgb.importance(model = bst1), regexp = NA) # no error
@@ -406,14 +449,28 @@ test_that("xgb.plot.importance de-duplicates features", {
 
 test_that("xgb.plot.tree works with and without feature names", {
   .skip_if_vcd_not_available()
-  expect_silent(xgb.plot.tree(feature_names = feature.names, model = bst.Tree.unnamed))
+  expect_silent(xgb.plot.tree(model = bst.Tree.unnamed))
   expect_silent(xgb.plot.tree(model = bst.Tree))
+
+  ## Categorical
+  y <- rnorm(100)
+  x <- sample(3, size = 100 * 3, replace = TRUE) |> matrix(nrow = 100)
+  x <- x - 1
+  dm <- xgb.DMatrix(data = x, label = y, nthread = 1)
+  setinfo(dm, "feature_type", c("c", "c", "c"))
+  model <- xgb.train(
+    data = dm,
+    params = list(tree_method = "hist"),
+    nrounds = 2
+  )
+  expect_silent(xgb.plot.tree(model = model))
 })
 
 test_that("xgb.plot.multi.trees works with and without feature names", {
   .skip_if_vcd_not_available()
-  xgb.plot.multi.trees(model = bst.Tree.unnamed, feature_names = feature.names, features_keep = 3)
+  xgb.plot.multi.trees(model = bst.Tree.unnamed, features_keep = 3)
   xgb.plot.multi.trees(model = bst.Tree, features_keep = 3)
+  expect_true(TRUE)
 })
 
 test_that("xgb.plot.deepness works", {
@@ -449,6 +506,26 @@ test_that("xgb.shap.data works with subsampling", {
   expect_equal(NROW(data_list$data), NROW(data_list$shap_contrib))
 })
 
+test_that("xgb.shap.data works with data frames", {
+  data(mtcars)
+  df <- mtcars
+  df$cyl <- factor(df$cyl)
+  x <- df[, -1]
+  y <- df$mpg
+  dm <- xgb.DMatrix(x, label = y, nthread = 1L)
+  model <- xgb.train(
+    data = dm,
+    params = list(
+      max_depth = 2,
+      nthread = 1
+    ),
+    nrounds = 2
+  )
+  data_list <- xgb.shap.data(data = df[, -1], model = model, top_n = 2, subsample = 0.8)
+  expect_equal(NROW(data_list$data), as.integer(0.8 * nrow(df)))
+  expect_equal(NROW(data_list$data), NROW(data_list$shap_contrib))
+})
+
 test_that("prepare.ggplot.shap.data works", {
   .skip_if_vcd_not_available()
   data_list <- xgb.shap.data(data = sparse_matrix, model = bst.Tree, top_n = 2)
@@ -472,21 +549,150 @@ test_that("xgb.plot.shap.summary works", {
   expect_silent(xgb.ggplot.shap.summary(data = sparse_matrix, model = bst.Tree, top_n = 2))
 })
 
+test_that("xgb.plot.shap.summary ignores categorical features", {
+  .skip_if_vcd_not_available()
+  data(mtcars)
+  df <- mtcars
+  df$cyl <- factor(df$cyl)
+  levels(df$cyl) <- c("a", "b", "c")
+  x <- df[, -1]
+  y <- df$mpg
+  dm <- xgb.DMatrix(x, label = y, nthread = 1L)
+  model <- xgb.train(
+    data = dm,
+    params = list(
+      max_depth = 2,
+      nthread = 1
+    ),
+    nrounds = 2
+  )
+  expect_warning({
+    xgb.ggplot.shap.summary(data = x, model = model, top_n = 2)
+  })
+
+  x_num <- mtcars[, -1]
+  x_num$gear <- as.numeric(x_num$gear) - 1
+  x_num <- as.matrix(x_num)
+  dm <- xgb.DMatrix(x_num, label = y, feature_types = c(rep("q", 8), "c", "q"), nthread = 1L)
+  model <- xgb.train(
+    data = dm,
+    params = list(
+      max_depth = 2,
+      nthread = 1
+    ),
+    nrounds = 2
+  )
+  expect_warning({
+    xgb.ggplot.shap.summary(data = x_num, model = model, top_n = 2)
+  })
+})
+
 test_that("check.deprecation works", {
-  ttt <- function(a = NNULL, DUMMY = NULL, ...) {
-    check.deprecation(...)
-    as.list((environment()))
-  }
-  res <- ttt(a = 1, DUMMY = 2, z = 3)
-  expect_equal(res, list(a = 1, DUMMY = 2))
-  expect_warning(
-    res <- ttt(a = 1, dummy = 22, z = 3)
-  , "\'dummy\' is deprecated")
-  expect_equal(res, list(a = 1, DUMMY = 22))
-  expect_warning(
-    res <- ttt(a = 1, dumm = 22, z = 3)
-  , "\'dumm\' was partially matched to \'dummy\'")
-  expect_equal(res, list(a = 1, DUMMY = 22))
+  data(mtcars)
+  dm <- xgb.DMatrix(mtcars[, -1L], label = mtcars$mpg, nthread = 1)
+  params <- xgb.params(nthread = 1, max_depth = 2, eval_metric = "rmse")
+  args_train <- list(
+    data = dm,
+    params = params,
+    nrounds = 10,
+    verbose = 0
+  )
+
+  # with exact name
+  options("xgboost.strict_mode" = TRUE)
+  expect_error({
+    model <- xgb.train(
+      data = dm,
+      params = params,
+      nrounds = 10,
+      watchlist = list(tr = dm),
+      verbose = 0
+    )
+  }, regexp = "watchlist")
+  options("xgboost.strict_mode" = FALSE)
+  expect_warning({
+    model <- xgb.train(
+      data = dm,
+      params = params,
+      nrounds = 10,
+      watchlist = list(tr = dm),
+      verbose = 0
+    )
+  }, regexp = "watchlist")
+  expect_true(hasName(attributes(model), "evaluation_log"))
+  expect_equal(names(attributes(model)$evaluation_log), c("iter", "tr_rmse"))
+
+  # with partial name match
+  expect_warning({
+    model <- xgb.train(
+      data = dm,
+      params = params,
+      nrounds = 10,
+      watchlis = list(train = dm),
+      verbose = 0
+    )
+  }, regexp = "watchlist")
+  expect_true(hasName(attributes(model), "evaluation_log"))
+  expect_equal(names(attributes(model)$evaluation_log), c("iter", "train_rmse"))
+
+  # error/warning is thrown if argument cannot be matched
+  options("xgboost.strict_mode" = TRUE)
+  expect_error({
+    model <- xgb.train(
+      data = dm,
+      params = params,
+      nrounds = 10,
+      watchlistt = list(train = dm),
+      verbose = 0
+    )
+  }, regexp = "unrecognized")
+  options("xgboost.strict_mode" = FALSE)
+  expect_warning({
+    model <- xgb.train(
+      data = dm,
+      params = params,
+      nrounds = 10,
+      watchlistt = list(train = dm),
+      verbose = 0
+    )
+  }, regexp = "unrecognized")
+
+  # error should suggest to put under 'params' if it goes there
+  options("xgboost.strict_mode" = TRUE)
+  expect_error({
+    model <- xgb.train(
+      data = dm,
+      nthread = 1, max_depth = 2, eval_metric = "rmse",
+      nrounds = 10,
+      evals = list(train = dm),
+      verbose = 0
+    )
+  }, regexp = "should be passed as a list to argument 'params'")
+  options("xgboost.strict_mode" = FALSE)
+  expect_warning({
+    model <- xgb.train(
+      data = dm,
+      nthread = 1, max_depth = 2, eval_metric = "mae",
+      nrounds = 10,
+      evals = list(train = dm),
+      verbose = 0
+    )
+  }, regexp = "should be passed as a list to argument 'params'")
+  expect_true(hasName(attributes(model), "evaluation_log"))
+  expect_equal(names(attributes(model)$evaluation_log), c("iter", "train_mae"))
+
+  # can take more than one deprecated parameter
+  expect_warning({
+    model <- xgb.train(
+      training.data = dm,
+      params = params,
+      nrounds = 10,
+      watchlis = list(tr = dm),
+      verbose = 0
+    )
+  }, regexp = "training.data")
+  expect_true(hasName(attributes(model), "evaluation_log"))
+  expect_equal(names(attributes(model)$evaluation_log), c("iter", "tr_rmse"))
 })
 
 test_that('convert.labels works', {
@@ -555,7 +761,7 @@ test_that("validate.features works as expected", {
     validate.features(model, as.matrix(mtcars[, 1:ncol(x)])) # nolint
   })
   expect_error({
-    validate.features(model, xgb.DMatrix(mtcars[, 1:3]))
+    validate.features(model, xgb.DMatrix(mtcars[, 1:3], nthread = 1))
   })
   expect_error({
     validate.features(model, as(x[, 1:3], "CsparseMatrix"))
@@ -563,15 +769,15 @@ test_that("validate.features works as expected", {
 
   # error when it cannot reorder or subset
   expect_error({
-    validate.features(model, xgb.DMatrix(mtcars))
+    validate.features(model, xgb.DMatrix(mtcars, nthread = 1))
   }, "Feature names")
   expect_error({
-    validate.features(model, xgb.DMatrix(x[, rev(colnames(x))]))
+    validate.features(model, xgb.DMatrix(x[, rev(colnames(x))], nthread = 1))
   }, "Feature names")
 
   # no error about types if the booster doesn't have types
   expect_error({
-    validate.features(model, xgb.DMatrix(x, feature_types = c(rep("q", 5), rep("c", 5))))
+    validate.features(model, xgb.DMatrix(x, feature_types = c(rep("q", 5), rep("c", 5)), nthread = 1))
   }, NA)
   tmp <- mtcars
   tmp[["vs"]] <- factor(tmp[["vs"]])
@@ -582,7 +788,7 @@ test_that("validate.features works as expected", {
   # error when types do not match
   setinfo(model, "feature_type", rep("q", 10))
   expect_error({
-    validate.features(model, xgb.DMatrix(x, feature_types = c(rep("q", 5), rep("c", 5))))
+    validate.features(model, xgb.DMatrix(x, feature_types = c(rep("q", 5), rep("c", 5)), nthread = 1))
   }, "Feature types")
   tmp <- mtcars
   tmp[["vs"]] <- factor(tmp[["vs"]])
@@ -590,3 +796,35 @@ test_that("validate.features works as expected", {
     validate.features(model, tmp)
   }, "Feature types")
 })
+
+test_that("Parameters constructor works as expected", {
+  empty_list <- list()
+  names(empty_list) <- character()
+
+  params <- xgb.params()
+  expect_equal(params, empty_list)
+
+  params <- xgb.params(max_depth = 2)
+  expect_equal(params, list(max_depth = 2))
+
+  params <- xgb.params(max_depth = NULL)
+  expect_equal(params, empty_list)
+
+  max_depth <- 3
+  params <- xgb.params(max_depth = max_depth)
+  expect_equal(params, list(max_depth = 3))
+
+  four <- 4L
+  params <- xgb.params(max_depth = four)
+  expect_equal(params, list(max_depth = 4L))
+
+  params <- xgb.params(objective = "binary:logistic", nthread = 10)
+  expect_equal(params, list(objective = "binary:logistic", nthread = 10))
+
+  expect_error({
+    xgb.params(max_xgboost = 10)
+  })
+  expect_error({
+    xgb.params(max_depth = 2, max_depth = 3)
+  })
+})
