@@ -10,13 +10,17 @@ dtest <- xgb.DMatrix(
   agaricus.test$data, label = agaricus.test$label, nthread = 2
 )
 
-bst <- xgb.train(data = dtrain,
-                 max_depth = 2,
-                 eta = 1,
-                 nrounds = 10,
-                 nthread = 1,
-                 verbose = 0,
-                 objective = "binary:logistic")
+bst <- xgb.train(
+  data = dtrain,
+  verbose = 0,
+  nrounds = 10,
+  params = xgb.params(
+    max_depth = 2,
+    learning_rate = 1,
+    nthread = 1,
+    objective = "binary:logistic"
+  )
+)
 
 test_that("call is exposed to R", {
   expect_false(is.null(attributes(bst)$call))
@@ -26,7 +30,7 @@ test_that("call is exposed to R", {
 test_that("params is exposed to R", {
   model_params <- attributes(bst)$params
   expect_is(model_params, "list")
-  expect_equal(model_params$eta, 1)
+  expect_equal(model_params$learning_rate, 1)
   expect_equal(model_params$max_depth, 2)
   expect_equal(model_params$objective, "binary:logistic")
 })
