@@ -5,6 +5,7 @@ import os
 from typing import (
     TYPE_CHECKING,
     Any,
+    AnyStr,
     Callable,
     Dict,
     List,
@@ -28,7 +29,10 @@ FeatureTypes = FeatureInfo
 BoosterParam = Union[List, Dict[str, Any]]  # better be sequence
 
 ArrayLike = Any
-PathLike = Union[str, os.PathLike]
+if TYPE_CHECKING:
+    PathLike = Union[str, os.PathLike[str]]
+else:
+    PathLike = Union[str, os.PathLike]
 CupyT = ArrayLike  # maybe need a stub for cupy arrays
 NumpyOrCupy = Any
 NumpyDType = Union[str, Type[np.number]]  # pylint: disable=invalid-name
@@ -48,7 +52,7 @@ FPreProcCallable = Callable
 # c_bst_ulong corresponds to bst_ulong defined in xgboost/c_api.h
 c_bst_ulong = ctypes.c_uint64  # pylint: disable=C0103
 
-ModelIn = Union[str, bytearray, os.PathLike]
+ModelIn = Union[os.PathLike[AnyStr], bytearray, str]
 
 CTypeT = TypeVar(
     "CTypeT",
