@@ -1,5 +1,5 @@
 /**
- * Copyright 2021-2023 by Contributors
+ * Copyright 2021-2024, XGBoost Contributors
  * \file row_set.h
  * \brief Quick Utility to compute subset of rows
  * \author Philip Cho, Tianqi Chen
@@ -11,14 +11,14 @@
 
 #include <algorithm>
 #include <cstddef>  // for size_t
-#include <limits>
 #include <memory>
 #include <utility>
 #include <vector>
 
-#include "../tree/hist/expand_entry.h"
+#include "../tree/sample_position.h"  // for SamplePosition
 #include "categorical.h"
 #include "column_matrix.h"
+#include "row_set.h"  // for RowSetCollection
 #include "xgboost/context.h"
 #include "xgboost/tree_model.h"
 
@@ -54,23 +54,23 @@ class PartitionBuilder {
   // Handle dense columns
   // Analog of std::stable_partition, but in no-inplace manner
   template <bool default_left, bool any_missing, typename ColumnType, typename Predicate>
-  inline std::pair<size_t, size_t> PartitionKernel(ColumnType* p_column,
-                                                   common::Span<const size_t> row_indices,
-                                                   common::Span<size_t> left_part,
-                                                   common::Span<size_t> right_part,
-                                                   size_t base_rowid, Predicate&& pred) {
+  std::pair<size_t, size_t> PartitionKernel(ColumnType* p_column,
+                                            common::Span<bst_idx_t const> row_indices,
+                                            common::Span<bst_idx_t> left_part,
+                                            common::Span<bst_idx_t> right_part,
+                                            bst_idx_t base_rowid, Predicate&& pred) {
     auto& column = *p_column;
-    size_t* p_left_part = left_part.data();
-    size_t* p_right_part = right_part.data();
-    size_t nleft_elems = 0;
-    size_t nright_elems = 0;
+    bst_idx_t* p_left_part = left_part.data();
+    bst_idx_t* p_right_part = right_part.data();
+    bst_idx_t nleft_elems = 0;
+    bst_idx_t nright_elems = 0;
 
     auto p_row_indices = row_indices.data();
     auto n_samples = row_indices.size();
 
     for (size_t i = 0; i < n_samples; ++i) {
       auto rid = p_row_indices[i];
-      const int32_t bin_id = column[rid - base_rowid];
+      bst_bin_t const bin_id = column[rid - base_rowid];
       if (any_missing && bin_id == ColumnType::kMissingId) {
         if (default_left) {
           p_left_part[nleft_elems++] = rid;
@@ -90,14 +90,14 @@ class PartitionBuilder {
   }
 
   template <typename Pred>
-  inline std::pair<size_t, size_t> PartitionRangeKernel(common::Span<const size_t> ridx,
-                                                        common::Span<size_t> left_part,
-                                                        common::Span<size_t> right_part,
+  inline std::pair<size_t, size_t> PartitionRangeKernel(common::Span<const bst_idx_t> ridx,
+                                                        common::Span<bst_idx_t> left_part,
+                                                        common::Span<bst_idx_t> right_part,
                                                         Pred pred) {
-    size_t* p_left_part = left_part.data();
-    size_t* p_right_part = right_part.data();
-    size_t nleft_elems = 0;
-    size_t nright_elems = 0;
+    bst_idx_t* p_left_part = left_part.data();
+    bst_idx_t* p_right_part = right_part.data();
+    bst_idx_t nleft_elems = 0;
+    bst_idx_t nright_elems = 0;
     for (auto row_id : ridx) {
       if (pred(row_id)) {
         p_left_part[nleft_elems++] = row_id;
@@ -112,10 +112,10 @@ class PartitionBuilder {
   void Partition(const size_t node_in_set, std::vector<ExpandEntry> const& nodes,
                  const common::Range1d range, const bst_bin_t split_cond,
                  GHistIndexMatrix const& gmat, const common::ColumnMatrix& column_matrix,
-                 const RegTree& tree, const size_t* rid) {
-    common::Span<const size_t> rid_span(rid + range.begin(), rid + range.end());
-    common::Span<size_t> left = GetLeftBuffer(node_in_set, range.begin(), range.end());
-    common::Span<size_t> right = GetRightBuffer(node_in_set, range.begin(), range.end());
+                 const RegTree& tree, bst_idx_t const* rid) {
+    common::Span<bst_idx_t const> rid_span{rid + range.begin(), rid + range.end()};
+    common::Span<bst_idx_t> left = GetLeftBuffer(node_in_set, range.begin(), range.end());
+    common::Span<bst_idx_t> right = GetRightBuffer(node_in_set, range.begin(), range.end());
     std::size_t nid = nodes[node_in_set].nid;
     bst_feature_t fid = tree.SplitIndex(nid);
     bool default_left = tree.DefaultLeft(nid);
@@ -184,8 +184,9 @@ class PartitionBuilder {
   }
 
   template <bool any_missing, typename ColumnType, typename Predicate>
-  void MaskKernel(ColumnType* p_column, common::Span<const size_t> row_indices, size_t base_rowid,
-                  BitVector* decision_bits, BitVector* missing_bits, Predicate&& pred) {
+  void MaskKernel(ColumnType* p_column, common::Span<bst_idx_t const> row_indices,
+                  bst_idx_t base_rowid, BitVector* decision_bits, BitVector* missing_bits,
+                  Predicate&& pred) {
     auto& column = *p_column;
     for (auto const row_id : row_indices) {
       auto const bin_id = column[row_id - base_rowid];
@@ -205,9 +206,9 @@ class PartitionBuilder {
   template <typename BinIdxType, bool any_missing, bool any_cat, typename ExpandEntry>
   void MaskRows(const size_t node_in_set, std::vector<ExpandEntry> const& nodes,
                 const common::Range1d range, bst_bin_t split_cond, GHistIndexMatrix const& gmat,
-                const common::ColumnMatrix& column_matrix, const RegTree& tree, const size_t* rid,
-                BitVector* decision_bits, BitVector* missing_bits) {
-    common::Span<const size_t> rid_span(rid + range.begin(), rid + range.end());
+                const common::ColumnMatrix& column_matrix, const RegTree& tree,
+                bst_idx_t const* rid, BitVector* decision_bits, BitVector* missing_bits) {
+    common::Span<bst_idx_t const> rid_span{rid + range.begin(), rid + range.end()};
     std::size_t nid = nodes[node_in_set].nid;
     bst_feature_t fid = tree.SplitIndex(nid);
     bool is_cat = tree.GetSplitTypes()[nid] == FeatureType::kCategorical;
@@ -263,11 +264,11 @@ class PartitionBuilder {
   template <typename ExpandEntry>
   void PartitionByMask(const size_t node_in_set, std::vector<ExpandEntry> const& nodes,
                        const common::Range1d range, GHistIndexMatrix const& gmat,
-                       const RegTree& tree, const size_t* rid, BitVector const& decision_bits,
+                       const RegTree& tree, bst_idx_t const* rid, BitVector const& decision_bits,
                        BitVector const& missing_bits) {
-    common::Span<const size_t> rid_span(rid + range.begin(), rid + range.end());
-    common::Span<size_t> left = GetLeftBuffer(node_in_set, range.begin(), range.end());
-    common::Span<size_t> right = GetRightBuffer(node_in_set, range.begin(), range.end());
+    common::Span<bst_idx_t const> rid_span(rid + range.begin(), rid + range.end());
+    common::Span<bst_idx_t> left = GetLeftBuffer(node_in_set, range.begin(), range.end());
+    common::Span<bst_idx_t> right = GetRightBuffer(node_in_set, range.begin(), range.end());
     std::size_t nid = nodes[node_in_set].nid;
     bool default_left = tree.DefaultLeft(nid);
 
@@ -299,12 +300,12 @@ class PartitionBuilder {
     }
   }
 
-  common::Span<size_t> GetLeftBuffer(int nid, size_t begin, size_t end) {
+  common::Span<bst_idx_t> GetLeftBuffer(int nid, size_t begin, size_t end) {
     const size_t task_idx = GetTaskIdx(nid, begin);
     return { mem_blocks_.at(task_idx)->Left(), end - begin };
   }
 
-  common::Span<size_t> GetRightBuffer(int nid, size_t begin, size_t end) {
+  common::Span<bst_idx_t> GetRightBuffer(int nid, size_t begin, size_t end) {
     const size_t task_idx = GetTaskIdx(nid, begin);
     return { mem_blocks_.at(task_idx)->Right(), end - begin };
   }
@@ -346,14 +347,14 @@ class PartitionBuilder {
     }
   }
 
-  void MergeToArray(int nid, size_t begin, size_t* rows_indexes) {
+  void MergeToArray(bst_node_t nid, size_t begin, bst_idx_t* rows_indexes) {
     size_t task_idx = GetTaskIdx(nid, begin);
 
-    size_t* left_result  = rows_indexes + mem_blocks_[task_idx]->n_offset_left;
-    size_t* right_result = rows_indexes + mem_blocks_[task_idx]->n_offset_right;
+    bst_idx_t* left_result = rows_indexes + mem_blocks_[task_idx]->n_offset_left;
+    bst_idx_t* right_result = rows_indexes + mem_blocks_[task_idx]->n_offset_right;
 
-    const size_t* left = mem_blocks_[task_idx]->Left();
-    const size_t* right = mem_blocks_[task_idx]->Right();
+    bst_idx_t const* left = mem_blocks_[task_idx]->Left();
+    bst_idx_t const* right = mem_blocks_[task_idx]->Right();
 
     std::copy_n(left, mem_blocks_[task_idx]->n_left, left_result);
     std::copy_n(right, mem_blocks_[task_idx]->n_right, right_result);
@@ -364,24 +365,23 @@ class PartitionBuilder {
   }
 
   // Copy row partitions into global cache for reuse in objective
-  template <typename Sampledp>
+  template <typename Invalidp>
   void LeafPartition(Context const* ctx, RegTree const& tree, RowSetCollection const& row_set,
-                     std::vector<bst_node_t>* p_position, Sampledp sampledp) const {
-    auto& h_pos = *p_position;
-    h_pos.resize(row_set.Data()->size(), std::numeric_limits<bst_node_t>::max());
-
+                     Span<bst_node_t> position, Invalidp invalidp) const {
     auto p_begin = row_set.Data()->data();
-    ParallelFor(row_set.Size(), ctx->Threads(), [&](size_t i) {
+    // For each node, walk through all the samples that fall in this node.
+    auto p_pos = position.data();
+    ParallelFor(row_set.Size(), ctx->Threads(), [&](auto i) {
       auto const& node = row_set[i];
       if (node.node_id < 0) {
         return;
       }
       CHECK(tree.IsLeaf(node.node_id));
-      if (node.begin) {  // guard for empty node.
-        size_t ptr_offset = node.end - p_begin;
+      if (node.begin()) {  // guard for empty node.
+        std::size_t ptr_offset = node.end() - p_begin;
         CHECK_LE(ptr_offset, row_set.Data()->size()) << node.node_id;
-        for (auto idx = node.begin; idx != node.end; ++idx) {
-          h_pos[*idx] = sampledp(*idx) ? ~node.node_id : node.node_id;
+        for (auto idx = node.begin(); idx != node.end(); ++idx) {
+          p_pos[*idx] = tree::SamplePosition::Encode(node.node_id, !invalidp(*idx));
         }
       }
     });
@@ -395,16 +395,16 @@ class PartitionBuilder {
     size_t n_offset_left;
     size_t n_offset_right;
 
-    size_t* Left() {
+    bst_idx_t* Left() {
       return &left_data_[0];
     }
 
-    size_t* Right() {
+    bst_idx_t* Right() {
       return &right_data_[0];
     }
    private:
-    size_t left_data_[BlockSize];
-    size_t right_data_[BlockSize];
+    bst_idx_t left_data_[BlockSize];
+    bst_idx_t right_data_[BlockSize];
   };
   std::vector<std::pair<size_t, size_t>> left_right_nodes_sizes_;
   std::vector<size_t> blocks_offsets_;
