@@ -32,5 +32,11 @@ inline std::string ObjTestNameGenerator(const ::testing::TestParamInfo<ParamType
   return name;
 };
 
-std::shared_ptr<DMatrix> MakeFmatForObjTest(std::string const& obj);
+/**
+ * @brief Construct random label for testing.
+ */
+void MakeLabelForObjTest(std::shared_ptr<DMatrix> p_fmat, std::string const& obj);
+
+std::shared_ptr<DMatrix> MakeFmatForObjTest(std::string const& obj, bst_idx_t n_samples,
+                                            bst_feature_t n_features);
 }  // namespace xgboost
