@@ -8,6 +8,10 @@ import xgboost as xgb
 from xgboost import testing as tm
 from xgboost.testing.data import check_inf
 from xgboost.testing.data_iter import run_mixed_sparsity
+from xgboost.testing.quantile_dmatrix import (
+    check_categorical_strings,
+    check_ref_quantile_cut,
+)
 
 sys.path.append("tests/python")
 import test_quantile_dmatrix as tqd
@@ -20,7 +24,7 @@ class TestQuantileDMatrix:
     def test_dmatrix_feature_weights(self) -> None:
         import cupy as cp
 
-        rng = cp.random.RandomState(1994)
+        rng = cp.random.RandomState(np.uint64(1994))
         data = rng.randn(5, 5)
         m = xgb.DMatrix(data)
 
@@ -32,6 +36,9 @@ class TestQuantileDMatrix:
             feature_weights.astype(np.float32),
         )
 
+    def test_categorical_strings(self) -> None:
+        check_categorical_strings("cuda")
+
     @pytest.mark.skipif(**tm.no_cupy())
     def test_dmatrix_cupy_init(self) -> None:
         import cupy as cp
@@ -142,11 +149,14 @@ class TestQuantileDMatrix:
                 {"tree_method": "approx", "max_bin": max_bin}, Xy, num_boost_round=4
             )
 
+    def test_ref_quantile_cut(self) -> None:
+        check_ref_quantile_cut("cuda")
+
     @pytest.mark.skipif(**tm.no_cupy())
     def test_metainfo(self) -> None:
         import cupy as cp
 
-        rng = cp.random.RandomState(1994)
+        rng = cp.random.RandomState(np.uint64(1994))
 
         rows = 10
         cols = 3
@@ -170,8 +180,8 @@ class TestQuantileDMatrix:
     def test_ref_dmatrix(self) -> None:
         import cupy as cp
 
-        rng = cp.random.RandomState(1994)
-        self.cputest.run_ref_dmatrix(rng, "gpu_hist", False)
+        rng = cp.random.RandomState(np.uint64(1994))
+        self.cputest.run_ref_dmatrix(rng, "cuda", False)
 
     @given(
         strategies.integers(1, 1000),
