@@ -1,6 +1,7 @@
 import json
 import os
 import tempfile
+from typing import Optional
 
 import numpy as np
 import pytest
@@ -17,38 +18,49 @@ rng = np.random.RandomState(1994)
 
 class TestModels:
     def test_glm(self):
-        param = {'objective': 'binary:logistic',
-                 'booster': 'gblinear', 'alpha': 0.0001, 'lambda': 1,
-                 'nthread': 1}
+        param = {
+            "objective": "binary:logistic",
+            "booster": "gblinear",
+            "alpha": 0.0001,
+            "lambda": 1,
+            "nthread": 1,
+        }
         dtrain, dtest = tm.load_agaricus(__file__)
-        watchlist = [(dtest, 'eval'), (dtrain, 'train')]
+        watchlist = [(dtest, "eval"), (dtrain, "train")]
         num_round = 4
         bst = xgb.train(param, dtrain, num_round, watchlist)
         assert isinstance(bst, xgb.core.Booster)
         preds = bst.predict(dtest)
         labels = dtest.get_label()
-        err = sum(1 for i in range(len(preds))
-                  if int(preds[i] > 0.5) != labels[i]) / float(len(preds))
+        err = sum(
+            1 for i in range(len(preds)) if int(preds[i] > 0.5) != labels[i]
+        ) / float(len(preds))
         assert err < 0.2
 
     def test_dart(self):
         dtrain, dtest = tm.load_agaricus(__file__)
-        param = {'max_depth': 5, 'objective': 'binary:logistic',
-                 'eval_metric': 'logloss', 'booster': 'dart', 'verbosity': 1}
+        param = {
+            "max_depth": 5,
+            "objective": "binary:logistic",
+            "eval_metric": "logloss",
+            "booster": "dart",
+            "verbosity": 1,
+        }
         # specify validations set to watch performance
-        watchlist = [(dtest, 'eval'), (dtrain, 'train')]
+        watchlist = [(dtest, "eval"), (dtrain, "train")]
         num_round = 2
         bst = xgb.train(param, dtrain, num_round, watchlist)
         # this is prediction
         preds = bst.predict(dtest, iteration_range=(0, num_round))
         labels = dtest.get_label()
-        err = sum(1 for i in range(len(preds))
-                  if int(preds[i] > 0.5) != labels[i]) / float(len(preds))
+        err = sum(
+            1 for i in range(len(preds)) if int(preds[i] > 0.5) != labels[i]
+        ) / float(len(preds))
         # error must be smaller than 10%
         assert err < 0.1
 
         with tempfile.TemporaryDirectory() as tmpdir:
-            dtest_path = os.path.join(tmpdir, 'dtest.dmatrix')
+            dtest_path = os.path.join(tmpdir, "dtest.dmatrix")
             model_path = os.path.join(tmpdir, "xgboost.model.dart.ubj")
             # save dmatrix into binary buffer
             dtest.save_binary(dtest_path)
@@ -66,28 +78,30 @@ class TestModels:
 
         def my_logloss(preds, dtrain):
             labels = dtrain.get_label()
-            return 'logloss', np.sum(
-                np.log(np.where(labels, preds, 1 - preds)))
+            return "logloss", np.sum(np.log(np.where(labels, preds, 1 - preds)))
 
         # check whether custom evaluation metrics work
-        bst = xgb.train(param, dtrain, num_round, watchlist,
-                        feval=my_logloss)
+        bst = xgb.train(
+            param, dtrain, num_round, evals=watchlist, custom_metric=my_logloss
+        )
         preds3 = bst.predict(dtest, iteration_range=(0, num_round))
         assert all(preds3 == preds)
 
         # check whether sample_type and normalize_type work
         num_round = 50
-        param['learning_rate'] = 0.1
-        param['rate_drop'] = 0.1
+        param["learning_rate"] = 0.1
+        param["rate_drop"] = 0.1
         preds_list = []
-        for p in [[p0, p1] for p0 in ['uniform', 'weighted']
-                  for p1 in ['tree', 'forest']]:
-            param['sample_type'] = p[0]
-            param['normalize_type'] = p[1]
-            bst = xgb.train(param, dtrain, num_round, watchlist)
+        for p in [
+            [p0, p1] for p0 in ["uniform", "weighted"] for p1 in ["tree", "forest"]
+        ]:
+            param["sample_type"] = p[0]
+            param["normalize_type"] = p[1]
+            bst = xgb.train(param, dtrain, num_round, evals=watchlist)
             preds = bst.predict(dtest, iteration_range=(0, num_round))
-            err = sum(1 for i in range(len(preds))
-                      if int(preds[i] > 0.5) != labels[i]) / float(len(preds))
+            err = sum(
+                1 for i in range(len(preds)) if int(preds[i] > 0.5) != labels[i]
+            ) / float(len(preds))
             assert err < 0.1
             preds_list.append(preds)
 
@@ -143,53 +157,67 @@ class TestModels:
         )
         assert booster.num_boosted_rounds() == 8
 
-    def run_custom_objective(self, tree_method=None):
+    def run_custom_objective(self, tree_method: Optional[str] = None):
         param = {
-            'max_depth': 2,
-            'eta': 1,
-            'objective': 'reg:logistic',
-            "tree_method": tree_method
+            "max_depth": 2,
+            "eta": 1,
+            "objective": "reg:logistic",
+            "tree_method": tree_method,
         }
         dtrain, dtest = tm.load_agaricus(__file__)
-        watchlist = [(dtest, 'eval'), (dtrain, 'train')]
+        watchlist = [(dtest, "eval"), (dtrain, "train")]
         num_round = 10
 
-        def logregobj(preds, dtrain):
-            labels = dtrain.get_label()
-            preds = 1.0 / (1.0 + np.exp(-preds))
-            grad = preds - labels
-            hess = preds * (1.0 - preds)
-            return grad, hess
-
-        def evalerror(preds, dtrain):
-            labels = dtrain.get_label()
-            preds = 1.0 / (1.0 + np.exp(-preds))
-            return 'error', float(sum(labels != (preds > 0.5))) / len(labels)
+        def evalerror(preds: np.ndarray, dtrain: xgb.DMatrix):
+            return tm.eval_error_metric(preds, dtrain, rev_link=True)
 
         # test custom_objective in training
-        bst = xgb.train(param, dtrain, num_round, watchlist, obj=logregobj,
-                        feval=evalerror)
-        assert isinstance(bst, xgb.core.Booster)
+        bst = xgb.train(
+            param,
+            dtrain,
+            num_round,
+            watchlist,
+            obj=tm.logregobj,
+            custom_metric=evalerror,
+        )
+        assert isinstance(bst, xgb.Booster)
         preds = bst.predict(dtest)
         labels = dtest.get_label()
-        err = sum(1 for i in range(len(preds))
-                  if int(preds[i] > 0.5) != labels[i]) / float(len(preds))
+        err = sum(
+            1 for i in range(len(preds)) if int(preds[i] > 0.5) != labels[i]
+        ) / float(len(preds))
         assert err < 0.1
 
         # test custom_objective in cross-validation
-        xgb.cv(param, dtrain, num_round, nfold=5, seed=0,
-               obj=logregobj, feval=evalerror)
+        xgb.cv(
+            param,
+            dtrain,
+            num_round,
+            nfold=5,
+            seed=0,
+            obj=tm.logregobj,
+            custom_metric=evalerror,
+        )
 
         # test maximize parameter
         def neg_evalerror(preds, dtrain):
             labels = dtrain.get_label()
-            return 'error', float(sum(labels == (preds > 0.0))) / len(labels)
+            preds = 1.0 / (1.0 + np.exp(-preds))
+            return "error", float(sum(labels == (preds > 0.0))) / len(labels)
 
-        bst2 = xgb.train(param, dtrain, num_round, watchlist, logregobj,
-                         neg_evalerror, maximize=True)
+        bst2 = xgb.train(
+            param,
+            dtrain,
+            num_round,
+            evals=watchlist,
+            obj=tm.logregobj,
+            custom_metric=neg_evalerror,
+            maximize=True,
+        )
         preds2 = bst2.predict(dtest)
-        err2 = sum(1 for i in range(len(preds2))
-                   if int(preds2[i] > 0.5) != labels[i]) / float(len(preds2))
+        err2 = sum(
+            1 for i in range(len(preds2)) if int(preds2[i] > 0.5) != labels[i]
+        ) / float(len(preds2))
         assert err == err2
 
     def test_custom_objective(self):
@@ -197,36 +225,54 @@ class TestModels:
 
     def test_multi_eval_metric(self):
         dtrain, dtest = tm.load_agaricus(__file__)
-        watchlist = [(dtest, 'eval'), (dtrain, 'train')]
-        param = {'max_depth': 2, 'eta': 0.2, 'verbosity': 1,
-                 'objective': 'binary:logistic'}
-        param['eval_metric'] = ["auc", "logloss", 'error']
+        watchlist = [(dtest, "eval"), (dtrain, "train")]
+        param = {
+            "max_depth": 2,
+            "eta": 0.2,
+            "verbosity": 1,
+            "objective": "binary:logistic",
+        }
+        param["eval_metric"] = ["auc", "logloss", "error"]
         evals_result = {}
-        bst = xgb.train(param, dtrain, 4, watchlist, evals_result=evals_result)
+        bst = xgb.train(param, dtrain, 4, evals=watchlist, evals_result=evals_result)
         assert isinstance(bst, xgb.core.Booster)
-        assert len(evals_result['eval']) == 3
-        assert set(evals_result['eval'].keys()) == {'auc', 'error', 'logloss'}
+        assert len(evals_result["eval"]) == 3
+        assert set(evals_result["eval"].keys()) == {"auc", "error", "logloss"}
 
     def test_fpreproc(self):
-        param = {'max_depth': 2, 'eta': 1, 'objective': 'binary:logistic'}
+        param = {"max_depth": 2, "eta": 1, "objective": "binary:logistic"}
         num_round = 2
 
         def fpreproc(dtrain, dtest, param):
             label = dtrain.get_label()
             ratio = float(np.sum(label == 0)) / np.sum(label == 1)
-            param['scale_pos_weight'] = ratio
+            param["scale_pos_weight"] = ratio
             return (dtrain, dtest, param)
 
         dtrain, _ = tm.load_agaricus(__file__)
-        xgb.cv(param, dtrain, num_round, nfold=5,
-               metrics={'auc'}, seed=0, fpreproc=fpreproc)
+        xgb.cv(
+            param,
+            dtrain,
+            num_round,
+            nfold=5,
+            metrics={"auc"},
+            seed=0,
+            fpreproc=fpreproc,
+        )
 
     def test_show_stdv(self):
-        param = {'max_depth': 2, 'eta': 1, 'objective': 'binary:logistic'}
+        param = {"max_depth": 2, "eta": 1, "objective": "binary:logistic"}
         num_round = 2
         dtrain, _ = tm.load_agaricus(__file__)
-        xgb.cv(param, dtrain, num_round, nfold=5,
-               metrics={'error'}, seed=0, show_stdv=False)
+        xgb.cv(
+            param,
+            dtrain,
+            num_round,
+            nfold=5,
+            metrics={"error"},
+            seed=0,
+            show_stdv=False,
+        )
 
     def test_prediction_cache(self) -> None:
         X, y = tm.make_sparse_regression(512, 4, 0.5, as_dense=False)
@@ -273,28 +319,34 @@ class TestModels:
             X = np.random.random((100, 30))
             y = np.random.randint(0, 4, size=(100,))
 
-            parameters['num_class'] = 4
+            parameters["num_class"] = 4
             m = xgb.DMatrix(X, y)
 
             booster = xgb.train(parameters, m)
-            dump = booster.get_dump(dump_format='json')
+            dump = booster.get_dump(dump_format="json")
 
             for i in range(len(dump)):
-                jsonschema.validate(instance=json.loads(dump[i]),
-                                    schema=schema)
+                jsonschema.validate(instance=json.loads(dump[i]), schema=schema)
 
         path = os.path.dirname(
-            os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
-        doc = os.path.join(path, 'doc', 'dump.schema')
-        with open(doc, 'r') as fd:
+            os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
+        )
+        doc = os.path.join(path, "doc", "dump.schema")
+        with open(doc, "r") as fd:
             schema = json.load(fd)
 
-        parameters = {'tree_method': 'hist', 'booster': 'gbtree',
-                      'objective': 'multi:softmax'}
+        parameters = {
+            "tree_method": "hist",
+            "booster": "gbtree",
+            "objective": "multi:softmax",
+        }
         validate_model(parameters)
 
-        parameters = {'tree_method': 'hist', 'booster': 'dart',
-                      'objective': 'multi:softmax'}
+        parameters = {
+            "tree_method": "hist",
+            "booster": "dart",
+            "objective": "multi:softmax",
+        }
         validate_model(parameters)
 
     def test_special_model_dump_characters(self) -> None:
@@ -363,7 +415,7 @@ class TestModels:
         sliced_trees = end * num_parallel_tree * num_classes
         assert sliced_trees == len(sliced.get_dump())
 
-        sliced = booster[: end]
+        sliced = booster[:end]
         sliced_trees = end * num_parallel_tree * num_classes
         assert sliced_trees == len(sliced.get_dump())
 
@@ -425,8 +477,8 @@ class TestModels:
         np.testing.assert_allclose(merged, single, atol=1e-6)
 
     @pytest.mark.skipif(**tm.no_sklearn())
-    @pytest.mark.parametrize("booster", ["gbtree", "dart"])
-    def test_slice(self, booster):
+    @pytest.mark.parametrize("booster_name", ["gbtree", "dart"])
+    def test_slice(self, booster_name: str) -> None:
         from sklearn.datasets import make_classification
 
         num_classes = 3
@@ -442,7 +494,7 @@ class TestModels:
                 "num_parallel_tree": num_parallel_tree,
                 "subsample": 0.5,
                 "num_class": num_classes,
-                "booster": booster,
+                "booster": booster_name,
                 "objective": "multi:softprob",
             },
             num_boost_round=num_boost_round,
@@ -452,6 +504,8 @@ class TestModels:
 
         assert len(booster.get_dump()) == total_trees
 
+        assert booster[...].num_boosted_rounds() == num_boost_round
+
         self.run_slice(
             booster, dtrain, num_parallel_tree, num_classes, num_boost_round, False
         )
