@@ -1,7 +1,6 @@
 import json
 import os
 import pickle
-import random
 import re
 import tempfile
 import warnings
@@ -337,6 +336,36 @@ def test_feature_importances_weight():
         cls.feature_importances_
 
 
+def test_feature_importances_weight_vector_leaf() -> None:
+    from sklearn.datasets import make_multilabel_classification
+
+    X, y = make_multilabel_classification(random_state=1994)
+    with pytest.raises(ValueError, match="gain/total_gain"):
+        clf = xgb.XGBClassifier(multi_strategy="multi_output_tree")
+        clf.fit(X, y)
+        clf.feature_importances_
+
+    with pytest.raises(ValueError, match="cover/total_cover"):
+        clf = xgb.XGBClassifier(
+            multi_strategy="multi_output_tree", importance_type="cover"
+        )
+        clf.fit(X, y)
+        clf.feature_importances_
+
+    clf = xgb.XGBClassifier(
+        multi_strategy="multi_output_tree",
+        importance_type="weight",
+        colsample_bynode=0.2,
+    )
+    clf.fit(X, y, feature_weights=np.arange(0, X.shape[1]))
+    fi = clf.feature_importances_
+    assert fi[0] == 0.0
+    assert fi[-1] > fi[1] * 5
+
+    w = np.polynomial.Polynomial.fit(np.arange(0, X.shape[1]), fi, deg=1)
+    assert w.coef[1] > 0.03
+
+
 @pytest.mark.skipif(**tm.no_pandas())
 def test_feature_importances_gain():
     from sklearn.datasets import load_digits
@@ -631,7 +660,7 @@ def test_sklearn_plotting():
     ax = xgb.plot_importance(classifier)
     assert isinstance(ax, Axes)
     assert ax.get_title() == 'Feature importance'
-    assert ax.get_xlabel() == 'F score'
+    assert ax.get_xlabel() == 'Importance score'
     assert ax.get_ylabel() == 'Features'
     assert len(ax.patches) == 4
 
@@ -1067,7 +1096,7 @@ def test_parameter_validation():
 def test_deprecate_position_arg():
     from sklearn.datasets import load_digits
     X, y = load_digits(return_X_y=True, n_class=2)
-    w = y
+    w = np.random.default_rng(0).uniform(size=y.size)
     with pytest.warns(FutureWarning):
         xgb.XGBRegressor(3, learning_rate=0.1)
     model = xgb.XGBRegressor(n_estimators=1)
@@ -1156,16 +1185,26 @@ def test_feature_weights(tree_method):
     for i in range(kCols):
         fw[i] *= float(i)
 
-    parser_path = os.path.join(tm.demo_dir(__file__), "json-model", "json_parser.py")
+    parser_path = os.path.join(tm.demo_dir(__file__), "guide-python", "model_parser.py")
     poly_increasing = get_feature_weights(
-        X, y, fw, parser_path, tree_method, xgb.XGBRegressor
+        X=X,
+        y=y,
+        fw=fw,
+        parser_path=parser_path,
+        tree_method=tree_method,
+        model=xgb.XGBRegressor,
     )
 
     fw = np.ones(shape=(kCols,))
     for i in range(kCols):
         fw[i] *= float(kCols - i)
     poly_decreasing = get_feature_weights(
-        X, y, fw, parser_path, tree_method, xgb.XGBRegressor
+        X=X,
+        y=y,
+        fw=fw,
+        parser_path=parser_path,
+        tree_method=tree_method,
+        model=xgb.XGBRegressor,
     )
 
     # Approxmated test, this is dependent on the implementation of random
@@ -1173,6 +1212,10 @@ def test_feature_weights(tree_method):
     assert poly_increasing[0] > 0.08
     assert poly_decreasing[0] < -0.08
 
+    reg = xgb.XGBRegressor(feature_weights=np.ones((kCols, )))
+    with pytest.raises(ValueError, match="Use the one in"):
+        reg.fit(X, y, feature_weights=np.ones((kCols, )))
+
 
 def run_boost_from_prediction_binary(tree_method, X, y, as_frame: Optional[Callable]):
     """
@@ -1563,3 +1606,16 @@ def test_sklearn_tags():
         except AttributeError as err:
             # only the exact error we expected to be raised should be raised
             assert bool(re.search(r"__sklearn_tags__.* should not be called", str(err)))
+
+
+def test_doc_link() -> None:
+    for est in [
+        xgb.XGBRegressor(),
+        xgb.XGBClassifier(),
+        xgb.XGBRanker(),
+        xgb.XGBRFRegressor(),
+        xgb.XGBRFClassifier(),
+    ]:
+        name = est.__class__.__name__
+        link = est._get_doc_link()
+        assert f"xgboost.{name}" in link
